// simple morse code detector for 6.200

void setup() {
  // put your setup code here, to run once:
  pinMode(A0, INPUT);
  pinMode(13, OUTPUT);
  Serial.begin(115200);
}

// BEGIN parameters (feel free to change these)
const float VOLTAGE_THRESHOLD = 0.75; // volts
const int EXPECTED_WPM = 10;          // how fast is the morse code input?
// END parameters

// additional parameters computed from the other ones
const int BIN_THRESHOLD = (int)(VOLTAGE_THRESHOLD / 3.3 * 1024);
const int DEBOUNCE_TIME = 20;
const int DOT_LENGTH = 1200 / EXPECTED_WPM;
const int DASH_LENGTH = 3 * DOT_LENGTH;
const int SPACE_LENGTH = 20 * DOT_LENGTH;

// lookup table
const char PUNCTUATION[] = "+-/=:.$?@&\"_";
const String SYMBOL_LOOKUP[] = {
    // letters
    ".-",   // A
    "-...", // B
    "-.-.", // C
    "-..",  // D
    ".",    // E
    "..-.", // F
    "--.",  // G
    "....", // H
    "..",   // I
    ".---", // J
    "-.-",  // K
    ".-..", // L
    "--",   // M
    "-.",   // N
    "---",  // O
    ".--.", // P
    "--.-", // Q
    ".-.",  // R
    "...",  // S
    "-",    // T
    "..-",  // U
    "...-", // V
    ".--",  // W
    "-..-", // X
    "-.--", // Y
    "--..", // Z

    // digits
    "-----", // 0
    ".----", // 1
    "..---", // 2
    "...--", // 3
    "....-", // 4
    ".....", // 5
    "-....", // 6
    "--...", // 7
    "---..", // 8
    "----.", // 9

    // punctuation
    ".-.-.",   // +
    "-....-",  // -
    "-..-.",   // /
    "-...-",   // =
    "---...",  // :
    ".-.-.-",  // .
    "...-..-", // $
    "..--..",  // ?
    ".--.-.",  // @
    ".-...",   // &
    ".-..-.",  // "
    "..--.-",  // _

    // if we get here, we've got an invalid code
    "ERROR",
};

String currentInput = "";
int debounceTime = 0;
int startTime = 0;
int stopTime = 0;
bool lastState = 0;
bool realState = 0;
bool newlineSent = 0;
bool awaitingSignal = 1;

void loop() {
  bool state = analogRead(A0) > BIN_THRESHOLD;
  digitalWrite(13, state); // set the Teensy's LED on/off

  int currentTime = millis();

  if (state != lastState) {
    debounceTime = currentTime;
  }
  lastState = state;

  if (currentTime - debounceTime > DEBOUNCE_TIME) {
    // after accounting for debounce, store some info
    realState = state;
    if (realState) {
      startTime = debounceTime;
      awaitingSignal = 0;
    } else {
      stopTime = debounceTime;
    }
  }

  if (realState) {
    // just clear this flag; we'll wait for things to turn off to actually do
    // anything
    newlineSent = false;
  } else {
    int pauseLength = currentTime - stopTime;
    // button is not currently pressed, several options here

    if (pauseLength > DOT_LENGTH / 2 && !awaitingSignal) {
      // paused more than half a dot, let's see what kind of symbol we got
      int signalLength = stopTime - startTime;
      if (signalLength < DASH_LENGTH / 2) {
        currentInput += ".";
      } else {
        currentInput += "-";
      }
      awaitingSignal = 1;
    }

    if (pauseLength > SPACE_LENGTH && !newlineSent) {
      // if paused for a long time, spit out a newline
      Serial.print("\n");
      newlineSent = true;
    } else if (pauseLength > DASH_LENGTH && currentInput != "") {
      // if pause longer than a dash, spit out a character and reset the buffer
      for (int i = 0; SYMBOL_LOOKUP[i] != "ERROR"; i++) {
        if (SYMBOL_LOOKUP[i] == currentInput) {
          if (i < 26) {
            // letters
            Serial.print(char('A' + i));
          } else if (i < 36) {
            // digits
            Serial.print(i - 26);
          } else {
            // punctuation
            Serial.print(PUNCTUATION[i - 36]);
          }
          break;
        }
      }
      currentInput = "";
    }
  }
}
